import itertools
import unittest
from cStringIO import StringIO
from statsd_ostools import parser

iostat_output = '''
Linux 2.6.32-220.el6.x86_64 (hostname)  10/04/2012  _x86_64_    (4 CPU)

avg-cpu:  %user   %nice %system %iowait  %steal   %idle
           5.35    0.00    1.17    0.24    0.00   93.24

Device:         rrqm/s   wrqm/s     r/s     w/s    rkB/s    wkB/s avgrq-sz avgqu-sz   await  svctm  %util
sdb               0.15    10.19    1.34    4.17    66.46    57.46    44.95     0.02    3.99   1.47   0.81
sda               0.27    72.78    0.36    1.44    20.40   296.87   353.29     0.32  177.34   2.82   0.51

avg-cpu:  %user   %nice %system %iowait  %steal   %idle
           0.48    0.00    0.96    0.00    0.00   98.56

Device:         rrqm/s   wrqm/s     r/s     w/s    rkB/s    wkB/s avgrq-sz avgqu-sz   await  svctm  %util
sdb               0.00     0.00    0.00   89.00     0.00   356.00     8.00     0.32    3.57   0.06   0.50
sda               0.00     0.00    0.00    0.00     0.00     0.00     0.00     0.00    0.00   0.00   0.00

avg-cpu:  %user   %nice %system %iowait  %steal   %idle
           1.64    0.00    1.41    0.47    0.00   96.48

Device:         rrqm/s   wrqm/s     r/s     w/s    rkB/s    wkB/s avgrq-sz avgqu-sz   await  svctm  %util
sdb               0.00    51.00    0.00   10.00     0.00   244.00    48.80     0.01    1.40   1.30   1.30
sda               0.00     7.00    0.00    4.00     0.00    44.00    22.00     0.01    1.50   0.50   0.20
'''.strip()

iostat_expected = [
    (
        (
            ('device', 'sdb'),
            ('rrqm/s', '0.00'),
            ('wrqm/s', '0.00'),
            ('r/s', '0.00'),
            ('w/s', '89.00'),
            ('rkB/s', '0.00'),
            ('wkB/s', '356.00'),
            ('avgrq-sz', '8.00'),
            ('avgqu-sz', '0.32'),
            ('await', '3.57'),
            ('svctm', '0.06'),
            ('%util', '0.50'),
        ),
        (
            ('device', 'sda'),
            ('rrqm/s', '0.00'),
            ('wrqm/s', '0.00'),
            ('r/s', '0.00'),
            ('w/s', '0.00'),
            ('rkB/s', '0.00'),
            ('wkB/s', '0.00'),
            ('avgrq-sz', '0.00'),
            ('avgqu-sz', '0.00'),
            ('await', '0.00'),
            ('svctm', '0.00'),
            ('%util', '0.00'),
        ),
    ),
    (
        (
            ('device', 'sdb'),
            ('rrqm/s', '0.00'),
            ('wrqm/s', '51.00'),
            ('r/s', '0.00'),
            ('w/s', '10.00'),
            ('rkB/s', '0.00'),
            ('wkB/s', '244.00'),
            ('avgrq-sz', '48.80'),
            ('avgqu-sz', '0.01'),
            ('await', '1.40'),
            ('svctm', '1.30'),
            ('%util', '1.30'),
        ),
        (
            ('device', 'sda'),
            ('rrqm/s', '0.00'),
            ('wrqm/s', '7.00'),
            ('r/s', '0.00'),
            ('w/s', '4.00'),
            ('rkB/s', '0.00'),
            ('wkB/s', '44.00'),
            ('avgrq-sz', '22.00'),
            ('avgqu-sz', '0.01'),
            ('await', '1.50'),
            ('svctm', '0.50'),
            ('%util', '0.20'),
        ),
    ),
]

mpstat_output = '''
Linux 2.6.32-220.el6.x86_64 (hostname)  10/04/2012  _x86_64_    (4 CPU)

03:46:30  CPU    %usr   %nice    %sys %iowait    %irq   %soft  %steal  %guest   %idle
03:46:31  all    0.50    0.00    0.25    0.00    0.00    0.00    0.00    0.00   99.25
03:46:31    0    2.02    0.00    1.01    0.00    0.00    0.00    0.00    0.00   96.97
03:46:31    1    0.99    0.00    0.99    0.00    0.00    0.00    0.00    0.00   98.02

03:46:31  CPU    %usr   %nice    %sys %iowait    %irq   %soft  %steal  %guest   %idle
03:46:32  all    0.50    0.00    0.50    0.00    0.00    0.00    0.00    0.00   98.99
03:46:32    0    1.04    0.00    0.00    0.00    0.00    0.00    0.00    0.00   98.96
03:46:32    1    0.00    0.00    0.00    0.00    0.00    0.00    0.00    0.00  100.00
'''.strip()

mpstat_expected = [
    (
        (
            ('CPU', 'all'),
            ('%usr', '0.50'),
            ('%nice', '0.00'),
            ('%sys', '0.25'),
            ('%iowait', '0.00'),
            ('%irq', '0.00'),
            ('%soft', '0.00'),
            ('%steal', '0.00'),
            ('%guest', '0.00'),
            ('%idle', '99.25'),
        ),
        (
            ('CPU', '0'),
            ('%usr', '2.02'),
            ('%nice', '0.00'),
            ('%sys', '1.01'),
            ('%iowait', '0.00'),
            ('%irq', '0.00'),
            ('%soft', '0.00'),
            ('%steal', '0.00'),
            ('%guest', '0.00'),
            ('%idle', '96.97'),
        ),
        (
            ('CPU', '1'),
            ('%usr', '0.99'),
            ('%nice', '0.00'),
            ('%sys', '0.99'),
            ('%iowait', '0.00'),
            ('%irq', '0.00'),
            ('%soft', '0.00'),
            ('%steal', '0.00'),
            ('%guest', '0.00'),
            ('%idle', '98.02'),
        ),
    ),
    (
        (
            ('CPU', 'all'),
            ('%usr', '0.50'),
            ('%nice', '0.00'),
            ('%sys', '0.50'),
            ('%iowait', '0.00'),
            ('%irq', '0.00'),
            ('%soft', '0.00'),
            ('%steal', '0.00'),
            ('%guest', '0.00'),
            ('%idle', '98.99'),
        ),
        (
            ('CPU', '0'),
            ('%usr', '1.04'),
            ('%nice', '0.00'),
            ('%sys', '0.00'),
            ('%iowait', '0.00'),
            ('%irq', '0.00'),
            ('%soft', '0.00'),
            ('%steal', '0.00'),
            ('%guest', '0.00'),
            ('%idle', '98.96'),
        ),
        (
            ('CPU', '1'),
            ('%usr', '0.00'),
            ('%nice', '0.00'),
            ('%sys', '0.00'),
            ('%iowait', '0.00'),
            ('%irq', '0.00'),
            ('%soft', '0.00'),
            ('%steal', '0.00'),
            ('%guest', '0.00'),
            ('%idle', '100.00'),
        ),
    ),
]

vmstat_output = '''
procs -----------memory---------- ---swap-- -----io---- --system-- -----cpu-----
 r  b   swpd   free   buff  cache   si   so    bi    bo   in   cs us sy id wa st
 0  0 188048 1240544 386980 1509204    0    0    22    88    1    1  5  1 93  0  0  
 0  0 188048 1240412 386988 1509204    0    0     0    36  355  399  1  0 99  0  0  
 0  0 188048 1240412 386988 1509204    0    0     0   196  280  342  0  0 99  0  0  
 0  0 188048 1240412 386988 1509204    0    0     0     0  225  275  0  1 99  0  0  
 0  0 188048 1240288 386988 1509204    0    0     0     0  722  887  1  1 98  0  0  
 0  0 188048 1240288 386988 1509204    0    0     0   332  387  390  0  0 99  1  0  
 1  0 188048 1238552 387064 1509216    0    0     0     0  924 1223  2  1 97  0  0  
procs -----------memory---------- ---swap-- -----io---- --system-- -----cpu-----
 r  b   swpd   free   buff  cache   si   so    bi    bo   in   cs us sy id wa st
 0  0 188048 1238552 387064 1509216    0    0     0     0 3054 3994  6  5 88  0  0  
 0  0 188048 1238552 387064 1509216    0    0     0     0  250  337  0  0 100  0  0 
 0  0 188048 1238552 387064 1509216    0    0     0     0  277  373  0  0 99  0  0  
'''.strip()

vmstat_expected = [
  (('r', '0'),
  ('b', '0'),
  ('swpd', '188048'),
  ('free', '1240544'),
  ('buff', '386980'),
  ('cache', '1509204'),
  ('si', '0'),
  ('so', '0'),
  ('bi', '22'),
  ('bo', '88'),
  ('in', '1'),
  ('cs', '1'),
  ('us', '5'),
  ('sy', '1'),
  ('id', '93'),
  ('wa', '0'),
  ('st', '0')),
 (('r', '0'),
  ('b', '0'),
  ('swpd', '188048'),
  ('free', '1240412'),
  ('buff', '386988'),
  ('cache', '1509204'),
  ('si', '0'),
  ('so', '0'),
  ('bi', '0'),
  ('bo', '36'),
  ('in', '355'),
  ('cs', '399'),
  ('us', '1'),
  ('sy', '0'),
  ('id', '99'),
  ('wa', '0'),
  ('st', '0')),
 (('r', '0'),
  ('b', '0'),
  ('swpd', '188048'),
  ('free', '1240412'),
  ('buff', '386988'),
  ('cache', '1509204'),
  ('si', '0'),
  ('so', '0'),
  ('bi', '0'),
  ('bo', '196'),
  ('in', '280'),
  ('cs', '342'),
  ('us', '0'),
  ('sy', '0'),
  ('id', '99'),
  ('wa', '0'),
  ('st', '0')),
 (('r', '0'),
  ('b', '0'),
  ('swpd', '188048'),
  ('free', '1240412'),
  ('buff', '386988'),
  ('cache', '1509204'),
  ('si', '0'),
  ('so', '0'),
  ('bi', '0'),
  ('bo', '0'),
  ('in', '225'),
  ('cs', '275'),
  ('us', '0'),
  ('sy', '1'),
  ('id', '99'),
  ('wa', '0'),
  ('st', '0')),
 (('r', '0'),
  ('b', '0'),
  ('swpd', '188048'),
  ('free', '1240288'),
  ('buff', '386988'),
  ('cache', '1509204'),
  ('si', '0'),
  ('so', '0'),
  ('bi', '0'),
  ('bo', '0'),
  ('in', '722'),
  ('cs', '887'),
  ('us', '1'),
  ('sy', '1'),
  ('id', '98'),
  ('wa', '0'),
  ('st', '0')),
 (('r', '0'),
  ('b', '0'),
  ('swpd', '188048'),
  ('free', '1240288'),
  ('buff', '386988'),
  ('cache', '1509204'),
  ('si', '0'),
  ('so', '0'),
  ('bi', '0'),
  ('bo', '332'),
  ('in', '387'),
  ('cs', '390'),
  ('us', '0'),
  ('sy', '0'),
  ('id', '99'),
  ('wa', '1'),
  ('st', '0')),
 (('r', '1'),
  ('b', '0'),
  ('swpd', '188048'),
  ('free', '1238552'),
  ('buff', '387064'),
  ('cache', '1509216'),
  ('si', '0'),
  ('so', '0'),
  ('bi', '0'),
  ('bo', '0'),
  ('in', '924'),
  ('cs', '1223'),
  ('us', '2'),
  ('sy', '1'),
  ('id', '97'),
  ('wa', '0'),
  ('st', '0')),
 (('r', 'r'),
  ('b', 'b'),
  ('swpd', 'swpd'),
  ('free', 'free'),
  ('buff', 'buff'),
  ('cache', 'cache'),
  ('si', 'si'),
  ('so', 'so'),
  ('bi', 'bi'),
  ('bo', 'bo'),
  ('in', 'in'),
  ('cs', 'cs'),
  ('us', 'us'),
  ('sy', 'sy'),
  ('id', 'id'),
  ('wa', 'wa'),
  ('st', 'st')),
 (('r', '0'),
  ('b', '0'),
  ('swpd', '188048'),
  ('free', '1238552'),
  ('buff', '387064'),
  ('cache', '1509216'),
  ('si', '0'),
  ('so', '0'),
  ('bi', '0'),
  ('bo', '0'),
  ('in', '3054'),
  ('cs', '3994'),
  ('us', '6'),
  ('sy', '5'),
  ('id', '88'),
  ('wa', '0'),
  ('st', '0')),
 (('r', '0'),
  ('b', '0'),
  ('swpd', '188048'),
  ('free', '1238552'),
  ('buff', '387064'),
  ('cache', '1509216'),
  ('si', '0'),
  ('so', '0'),
  ('bi', '0'),
  ('bo', '0'),
  ('in', '250'),
  ('cs', '337'),
  ('us', '0'),
  ('sy', '0'),
  ('id', '100'),
  ('wa', '0'),
  ('st', '0')),
 (('r', '0'),
  ('b', '0'),
  ('swpd', '188048'),
  ('free', '1238552'),
  ('buff', '387064'),
  ('cache', '1509216'),
  ('si', '0'),
  ('so', '0'),
  ('bi', '0'),
  ('bo', '0'),
  ('in', '277'),
  ('cs', '373'),
  ('us', '0'),
  ('sy', '0'),
  ('id', '99'),
  ('wa', '0'),
  ('st', '0'))
]

class TestParsers(unittest.TestCase):
    def test_parsers(self):
        self.maxDiff = None
        todo = (
            (parser.IOStatParser, iostat_output, iostat_expected),
            (parser.MPStatParser, mpstat_output, mpstat_expected),
            (parser.VMStatParser, vmstat_output, vmstat_expected),
        )
        for parserklass, output, expected in todo:
            data = list(parserklass(StringIO(output)))
            self.assertEqual(expected, data)

if __name__ == '__main__':
    unittest.main()
